#include "JSVRoboBilly.h"

//edit 12-09-2018:
//switched Pin1 and Pin2 of the motorcircuits because the use of the tone function interferes with PWM on pin 11 (see arduino forums)
//By using Pin11 just as a digital output this interference is avoided, in the code only Pin1 recieves the analogWrite instruction

//edit 20-09-2018
//switched pins back again because pin 10 and 9 cant be used in conjunction with the servo.h library

JSVRoboBilly::JSVRoboBilly(){
	//setting pins
	pinMode(MotorLeftPin1,OUTPUT);
	pinMode(MotorLeftPin2,OUTPUT);
	pinMode(MotorRightPin1,OUTPUT);
	pinMode(MotorRightPin2,OUTPUT);
	LeftMotor = JSVMotor(MotorLeftPin1,MotorLeftPin2,MinMotorSpeed,MaxMotorSpeed);
	RightMotor = JSVMotor(MotorRightPin1,MotorRightPin2,MinMotorSpeed,MaxMotorSpeed);
	pinMode(TrigPin,OUTPUT);
	pinMode(EchoPin,INPUT);
	pinMode(LeftLEDPin,OUTPUT);
	pinMode(RightLEDPin,OUTPUT);
	pinMode(SpeakerPin,OUTPUT);
	//CANNOT INITIALISE SERVO HERE DUE TO POINTERS AND TIMERS AND SO ON.... (SEE FORUMS) trust me I spent 2 hours figuring this out!
	//LeftArm.attach(LeftArmServoPin);
	//RightArm.attach(RightArmServoPin);
  	Sonar = NewPing(TrigPin,EchoPin,MaxDistanceCm);
}

//MOTORS
void JSVRoboBilly::control_motor(int index, bool MotorDirection, uint8_t Percentage){
  if (Percentage > 100){
    Percentage = 100;
  }
  if (index==0){
	LeftMotor.ControlMotor(MotorDirection,Percentage);
  }
  if (index==1){
  RightMotor.ControlMotor(MotorDirection,Percentage);
  }
}
void JSVRoboBilly::SetupLeftMotor(bool ReverseMotor, int Percentage){
 	if (Percentage > 100){
    	Percentage = 100;
  	}
  	if (ReverseMotor){
		LeftMotor.ReverseMotor();	
	}
	LeftMotor.SetMaxMotorSpeed(MaxMotorSpeed*Percentage/100);
  	LeftMotor.SetMinMotorSpeed(MinMotorSpeed*Percentage/100);
}

void JSVRoboBilly::SetupRightMotor(bool ReverseMotor, int Percentage){
 	if (Percentage > 100){
    	Percentage = 100;
  	}
  	if (ReverseMotor){
		RightMotor.ReverseMotor();	
	}
	RightMotor.SetMaxMotorSpeed(MaxMotorSpeed*Percentage/100);
  	RightMotor.SetMinMotorSpeed(MinMotorSpeed*Percentage/100);
}

//ULTRASONIC SENSOR

unsigned int JSVRoboBilly::getDistance(){
    return Sonar.convert_cm(Sonar.ping_median(PingQuality));
}

//LED

void JSVRoboBilly::LeftLEDOn(){
	digitalWrite(LeftLEDPin,HIGH);
}
void JSVRoboBilly::LeftLEDOff(){
	digitalWrite(LeftLEDPin,LOW);
}
void JSVRoboBilly::RightLEDOn(){
	digitalWrite(RightLEDPin,HIGH);
}
void JSVRoboBilly::RightLEDOff(){
	digitalWrite(RightLEDPin,LOW);
}
void JSVRoboBilly::LeftLEDSet(bool set){
	if (set) LeftLEDOn();
	else LeftLEDOff();
}
void JSVRoboBilly::RightLEDSet(bool set){
	if (set) RightLEDOn();
	else RightLEDOff();
}
void JSVRoboBilly::LeftLEDToggle(){
	if (!digitalRead(LeftLEDPin)) LeftLEDOn();
	else LeftLEDOff();
}
void JSVRoboBilly::RightLEDToggle(){
	if (!digitalRead(RightLEDPin)) RightLEDOn();
	else RightLEDOff();
}

//SPEAKER

void JSVRoboBilly::SpeakerPlayNote(int frequency){
	tone(SpeakerPin,frequency);
}
void JSVRoboBilly::SpeakerPlayNoteTime(int frequency, int time_ms){
	tone(SpeakerPin,frequency,time_ms);
	delay(time_ms);
}
void JSVRoboBilly::SpeakerStop(){
	noTone(SpeakerPin);
}

//BLUETOOTH

void JSVRoboBilly::BluetoothOpenConnection(){
	Serial.begin(BluetoothBaudRate);
}
bool JSVRoboBilly::BluetoothHasData(){
	return Serial.available();
}
char JSVRoboBilly::BluetoothGetData(){
	return Serial.read();
}
void JSVRoboBilly::BluetoothSendData(String data){
	for (int i = 0; i < data.length(); i++)
  {
    Serial.write(data[i]);   // Push each char 1 by 1 on each loop pass
  }
}

//Servo
void JSVRoboBilly::InitServos(){
  LeftArm.attach(LeftArmServoPin);
  //RightArm.attach(RichtArmServoPin);
}

void JSVRoboBilly::SetServoAngle(Servo servomotor, int angle){
	servomotor.write(angle);
}
void JSVRoboBilly::SetLeftArmAngle(int angle){
	SetServoAngle(LeftArm,angle);
}
void JSVRoboBilly::SetRightArmAngle(int angle){
	SetServoAngle(RightArm,angle);
}
int JSVRoboBilly::ReadServoAngle(Servo servomotor){
  return servomotor.read();
}
int JSVRoboBilly::ReadLeftArmAngle(){
	return ReadServoAngle(LeftArm);
}
int JSVRoboBilly::ReadRightArmAngle(){
	return ReadServoAngle(RightArm);
}
